SystemOrganization addCategory: #'Zinc-AWS'!

Object subclass: #ZnAWSS3Client
	instanceVariableNames: 'accessKeyId secretAccessKey fixedClient checkIntegrity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-AWS'!

!ZnAWSS3Client commentStamp: 'SvenVanCaekenberghe 1/5/2011 20:01' prior: 0!
I am ZnAWSS3Client, a basic AWS S3 Client (http://en.wikipedia.org/wiki/Amazon_S3).

| client |
(client := ZnAWSS3Client new)
	accessKeyId: '2ZGSSBGBHQGJ9VV5N441';
	secretAccessKey: 'OtxrzxIsfpFjA7SwPzILwy8Bw21TLhquhboDYROV';
	checkIntegrity: true.
client buckets.
client keysIn: 'my-bucket'.
client at: 'my-bucket' -> 'my-key'.
client at: 'my-bucket' -> 'my-key' put: (ZnEntity with: '0123456789').
client at: 'my-bucket' -> 'my-key' put: (ZnEntity with: 'Smalltalk rules S3!!') headers: (Dictionary with: 'x-amz-acl'->'public-read').

I depend on Zinc HTTP Components and XML Support.!

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:07'!
accessKeyId
	"The AWS Security Credentials' Access Keys' Access Key ID used by me"
	
	^ accessKeyId! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:08'!
accessKeyId: string
	"Set my AWS Security Credentials' Access Keys' Access Key ID"

	accessKeyId := string! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 11:28'!
addAuthorizationTo: request
	| authorization |
	authorization := self authorizationFor: request.
	request headers 
		at: 'Authorization' 
		put: (String streamContents: [ :stream |
				stream nextPutAll: 'AWS '; nextPutAll: self accessKeyId; nextPut: $:; nextPutAll: authorization ]).


! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:10'!
addBucket: bucket
	"Create a new AWS S3 bucket"

	| url request |
	url := self asUrl: bucket.
	request := ZnRequest put: url.
	^ self executeRequest: request
! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 17:04'!
addContentHashTo: request
	| md5 |
	request hasEntity ifFalse: [ ^ self ].
	md5 := self md5: request contents.
	request headers at: 'Content-MD5' put: md5 base64Encoded


! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 11:16'!
asResource: bucketKeyAssociation
	^ String streamContents: [ :stream |
		stream 
			nextPut: $/; 
			nextPutAll: bucketKeyAssociation key; 
			nextPut: $/; 
			nextPutAll: bucketKeyAssociation value ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 11:17'!
asUrl: resource
	^ self fixedClient fixedUrl: resource! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:15'!
at: bucketKeyAssociation
	"Access an AWS S3 object in a bucket,
	identified by a bucket->key association"
	
	| url request |
	url := self asUrl: (self asResource: bucketKeyAssociation).
	request := ZnRequest get: url.
	^ self executeRequest: request! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:16'!
at: bucketKeyAssociation put: object
	"Set an AWS S3 object in a bucket to object,
	assumed to be ZnEntity compatible,
	identified by a bucket->key association"

	^ self at: bucketKeyAssociation put: object headers: nil! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:16'!
at: bucketKeyAssociation put: object headers: headers
	"Set an AWS S3 object in a bucket to object,
	assumed to be ZnEntity compatible,
	identified by a bucket->key association
	adding headers to the request"
	
	| url request |
	url := self asUrl: (self asResource: bucketKeyAssociation).
	(request := ZnRequest put: url)
		entity: object.
	headers notNil 
		ifTrue: [ request headers addAll: headers ].
	^ self executeRequest: request! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 22:57'!
authorizationFor: request
	| canonicalString hmac |
	request headers at: 'Date' put: ZnUtils httpDate.
	canonicalString := self canonicalStringFor: request.
	hmac := self hmacSha1: canonicalString.
	^ hmac base64Encoded ! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:17'!
buckets
	"Return an array of all  AWS S3 bucket name strings for the account"
	
	| url request response |
	url := self asUrl: '/'.
	request := ZnRequest get: url.
	response := self executeRequest: request.
	^ (response isSuccess and: [ response contentType = ZnMimeType applicationXml ])
		ifTrue: [ | dom |
			dom := XMLDOMParser parse: response contents.
			((dom allElementsNamed: 'Bucket') collect: [ :each | 
				each contentStringAt: 'Name']) asArray ]
		ifFalse: [
			self error: 'Could not list buckets' ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 19:36'!
canonicalStringFor: request
	^ String streamContents: [ :stream |
		stream nextPutAll: request method; lf.
		(request headers includesKey: 'Content-MD5')
			ifTrue: [ stream nextPutAll: (request headers at: 'Content-MD5') ].
		stream lf.
		request headers hasContentType ifTrue: [
			stream print: request headers contentType ].
		stream lf.
		(request headers includesKey: 'Date')
			ifTrue: [ stream nextPutAll: (request headers at: 'Date') ].
		stream lf.
		self printAmzHeaders: request headers to: stream.
		stream nextPutAll: request uri pathPrintString ]! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:09'!
checkIntegrity
	"Do I check the integrety of get/put requests using content hashes ?"
	
	checkIntegrity isNil ifTrue: [ checkIntegrity := false ].
	^ checkIntegrity! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:09'!
checkIntegrity: boolean
	"Set whether I should check the integrety of get/put requests using content hashes"

	checkIntegrity := boolean! !

!ZnAWSS3Client methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2011 20:05'!
close
	"Close the network resources we are using"
	
	fixedClient notNil 
		ifTrue: [ fixedClient close ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 16:51'!
enforceContentHashFor: request
	| etag md5 |
	request contentLength = 0 ifTrue: [ ^ self ].
	etag := (request headers at: 'ETag' ifAbsent: [ ^ self ]) withoutQuoting.
	md5 := self md5: request contents.
	etag = md5 hex
		ifFalse: [ self error: 'Content MD5 hash and ETag do not match' ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 16:49'!
executeRequest: request
	| response |
	self prepareRequest: request.
	response := self fixedClient executeRequest: request.
	self postProcessResponse: response.
	^ response

! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 11:12'!
fixedClient
	fixedClient isNil ifTrue: [ self initializeFixedClient ].
	^ fixedClient! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 22:42'!
hmacSha1: string
	^ (HMAC on: SHA1 new)
		key: self secretAccessKey asByteArray;
		digestMessage: string asByteArray! !

!ZnAWSS3Client methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2011 11:13'!
initializeFixedClient
	fixedClient := ZnFixedClient host: 's3.amazonaws.com'! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:14'!
keysIn: bucket
	"Return a collection of ZnAWSS3Key objects describing the keys in bucket"
	
	| url request response |
	url := self asUrl: bucket.
	request := ZnRequest get: url.
	response := self executeRequest: request.
	^ (response isSuccess and: [ response contentType = ZnMimeType applicationXml ])
		ifTrue: [ | dom |
			dom := XMLDOMParser parse: response contents.
			((dom allElementsNamed: 'Contents') collect: [ :each | 
				(ZnAWSS3Key fromXml: each) bucket: bucket; yourself ]) asArray ]
		ifFalse: [
			self error: 'Could not list keys' ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 11:30'!
md5: object
	^ MD5 new hashMessage: object! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 16:50'!
postProcessResponse: request
	self checkIntegrity 
		ifTrue: [ self enforceContentHashFor: request ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 16:54'!
prepareRequest: request
	self checkIntegrity 
		ifTrue: [ self addContentHashTo: request ].
	self addAuthorizationTo: request.
	

! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 19:42'!
printAmzHeaders: headers to: stream
	| amzHeaders |
	amzHeaders := OrderedCollection new.
	headers headersDo: [ :key :value |
		(key beginsWith: 'X-Amz')
			ifTrue: [ 
				amzHeaders add: key asLowercase -> value ] ].
	amzHeaders sorted do: [ :each |
		stream 
			nextPutAll: each key; 
			nextPut: $:; 
			nextPutAll: each value; 
			lf ]! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:17'!
remove: bucketKeyAssociation
	"Remove an  AWS S3 object in a bucket,
	identified by a bucket->key association"

	| url request |
	url := self asUrl: (self asResource: bucketKeyAssociation).
	request := ZnRequest delete: url.
	^ self executeRequest: request! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:10'!
removeBucket: bucket
	"Remove an AWS S3 bucket"
	
	| url request |
	url := self asUrl: bucket.
	request := ZnRequest delete: url.
	^ self executeRequest: request
! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:07'!
secretAccessKey
	"The AWS Security Credentials' Access Keys' Secret Access Key used by me"
	
	^ secretAccessKey! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:07'!
secretAccessKey: string
	"Set my AWS Security Credentials' Access Keys' Secret Access Key"

	secretAccessKey := string! !

Object subclass: #ZnAWSS3Key
	instanceVariableNames: 'key size etag bucket xml bucketKeyAssociation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-AWS'!

!ZnAWSS3Key commentStamp: 'SvenVanCaekenberghe 1/5/2011 16:11' prior: 0!
I am ZnAWSS3Key.

I hold each result from listing the keys in an AWS S3 bucket.

Use #value to convert me to what ZnAWSS3Client>>#at: 
and ZnAWSS3Client>>#at:put: expect (a bucket->key association).!

!ZnAWSS3Key class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/5/2011 15:21'!
fromXml: dom
	^ self new fromXml: dom; yourself! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
bucket
	^ bucket! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
bucket: anObject
	bucket := anObject! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:37'!
bucketKeyAssociation
	bucketKeyAssociation isNil ifTrue: [ 
		bucketKeyAssociation := bucket -> key ].
	^ bucketKeyAssociation! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
etag
	^ etag! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
etag: anObject
	etag := anObject! !

!ZnAWSS3Key methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2011 15:31'!
fromXml: dom
	self 
		key: (dom contentStringAt: 'Key');
		etag: (dom contentStringAt: 'ETag') withoutQuoting; 
		size: (Integer readFrom: (dom contentStringAt: 'Size') ifFail: [ -1 ]); 
		xml: dom
! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
key
	^ key! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
key: anObject
	key := anObject! !

!ZnAWSS3Key methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:37'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream print: self bucketKeyAssociation.
	stream nextPut: $)! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:25'!
size
	^ size! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
size: anObject
	size := anObject! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:43'!
value
	^ self bucketKeyAssociation! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
xml
	^ xml! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
xml: anObject
	xml := anObject! !
